﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class NormalDoubleValueManipulator : SingleSuccessorOperator, IDoubleValueManipulator, IStochasticOperator {

    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public NormalDoubleValueManipulator() { }
    [StorableConstructor]
    protected NormalDoubleValueManipulator(bool deserializing) : base(deserializing) { }
    protected NormalDoubleValueManipulator(NormalDoubleValueManipulator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NormalDoubleValueManipulator(this, cloner);
    }
    
    // todo: override apply
    public void Apply(IRandom random, DoubleValue value, DoubleValueRange range) {
      ApplyStatic(random, value, range);
    }

    public static void ApplyStatic(IRandom random, DoubleValue value, DoubleValueRange range) {
      var strategy = new RealVector(new double[] { (range.UpperBound.Value - range.LowerBound.Value) / 10}); // todo: add strategy parameter
      var vector = new RealVector(1);
      double val = value.Value;
      
      do { 
        vector[0] = val;
        NormalAllPositionsManipulator.Apply(random, vector, strategy);
        value.Value = vector[0];
        value.Value = range.ApplyStepSize(value.Value);
      } while(!range.IsInRange(value.Value));
    }
  }
}